import sys
import os

from .configuration import ConfigManager, ConfigurationScheme
from .version import VERSION


class LoggingConfiguration(ConfigurationScheme):
    LOGGER_NAME = "rook"
    FILE_NAME = "rookout/python-rook.log"
    LOG_TO_STDERR = False
    LOG_LEVEL = "INFO"
    PROPAGATE_LOGS = False
    LOG_TO_REMOTE = False
    DEBUG = False


class VersionConfiguration(ConfigurationScheme):
    VERSION = VERSION
    COMMIT = "CommitGoesHere"


class DefaultConfiguration(ConfigurationScheme):
    DEFAULT_CONFIG_FILE_NAME = "rookout-config.json"
    LOAD_FROM_APP_FOLDER = True
    IGNORE_FILE = "ignore-rookout-config"
    USER_CONFIGURATION_FILE = "rookout/rook-config.json"


class ErrorConfiguration(ConfigurationScheme):
    STACK_DEPTH = 10


class ProtobufConfiguration(ConfigurationScheme):
    NAMESPACE_SERIALIZER_DUMPING = r""


class ControllerAddress(ConfigurationScheme):
    HOST = 'wss://control.rookout.com'
    PORT = 443


class AgentComConfiguration(ConfigurationScheme):
    COMMAND_THREAD_NAME = "rookout_agent_com"
    MAX_MESSAGE_LENGTH = 100 * 1024 * 1024
    BACK_OFF = 0.2
    MAX_SLEEP = 60
    TIMEOUT = 2
    REQUEST_TIMEOUT_SECS = 30
    PING_TIMEOUT = 30
    PING_INTERVAL = 10
    RESET_BACKOFF_TIMEOUT = 3*60.0
    FLUSH_TIMEOUT = 3


class OutputConfiguration(ConfigurationScheme):
    FLUSH_TIME_INTERVAL = 0.25
    THREAD_NAME = "rookout_output_thread"
    MAX_ITEMS = 40
    MAX_LOG_ITEMS = 100
    MAX_STATUS_UPDATES = 100


class OutputWsConfiguration(ConfigurationScheme):
    MAX_STATUS_UPDATES = 200
    MAX_LOG_ITEMS = 200
    MAX_AUG_MESSAGES = 100
    BUCKET_REFRESH_RATE = 10


class InstrumentationConfig(ConfigurationScheme):
    ENGINE = "auto"
    MIN_TIME_BETWEEN_HITS_MS = 100
    MAX_AUG_TIME = 400


class ImportServiceConfig(ConfigurationScheme):
    USE_IMPORT_HOOK = True
    SYS_MODULES_QUERY_INTERVAL = 0.25
    THREAD_NAME = "rookout_import_service_thread"


class HttpServerServiceConfig(ConfigurationScheme):
    SERVICES_NAMES = ""


class GitConfig(ConfigurationScheme):
    GIT_COMMIT = None
    GIT_ORIGIN = None


manager = ConfigManager(sys.modules[__name__])


def _load_config_file():
    # Calculate package path
    if 'rook' in __name__:
        # Find the rook directory
        import rook
        package_path = rook.__path__[0]
    elif sys.argv[0].endswith('/rookout-agent.py') or sys.argv[0].endswith('rookout-agent'):
        try:
            import agent
            package_path = agent.__path__[0]
        except ImportError:
            pass
    else:
        package_path = None

    # If a package configuration file exists, load it
    if package_path:
        package_config = os.path.join(package_path, DefaultConfiguration.DEFAULT_CONFIG_FILE_NAME)
        manager.safe_load_json_file(package_config)

    # If a user configuration file exists, load it
    if DefaultConfiguration.USER_CONFIGURATION_FILE:

        # Get full path
        if os.path.isabs(DefaultConfiguration.USER_CONFIGURATION_FILE):
            user_config_path = DefaultConfiguration.USER_CONFIGURATION_FILE
        elif "darwin" in sys.platform:
            user_config_path = os.path.join(os.getenv("HOME", "."), DefaultConfiguration.USER_CONFIGURATION_FILE)
        elif sys.platform == "win32":
            user_config_path = os.path.join(os.getenv("USERPROFILE", "."), DefaultConfiguration.USER_CONFIGURATION_FILE)
        else:
            user_config_path = os.path.join("/etc", DefaultConfiguration.USER_CONFIGURATION_FILE)

        manager.safe_load_json_file(user_config_path)

    # If an app configuration file exists, load it
    if DefaultConfiguration.LOAD_FROM_APP_FOLDER:
        app_path = os.path.dirname(os.path.abspath(sys.argv[0]))
        if app_path == package_path:
            return

        if DefaultConfiguration.IGNORE_FILE and os.path.exists(
                os.path.join(app_path, DefaultConfiguration.IGNORE_FILE)):
            return

        app_config = os.path.join(app_path, DefaultConfiguration.DEFAULT_CONFIG_FILE_NAME)
        manager.safe_load_json_file(app_config)


_load_config_file()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: variant.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='variant.proto',
  package='com.rookout',
  syntax='proto3',
  serialized_pb=_b('\n\rvariant.proto\x12\x0b\x63om.rookout\x1a\x1fgoogle/protobuf/timestamp.proto\"\x9c\x01\n\x05\x45rror\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12(\n\nparameters\x18\x03 \x01(\x0b\x32\x14.com.rookout.Variant\x12!\n\x03\x65xc\x18\x04 \x01(\x0b\x32\x14.com.rookout.Variant\x12\'\n\ttraceback\x18\x05 \x01(\x0b\x32\x14.com.rookout.Variant\"\x8a\x10\n\x07Variant\x12/\n\x0cvariant_type\x18\x01 \x01(\x0e\x32\x19.com.rookout.Variant.Type\x12\x15\n\roriginal_type\x18\x02 \x01(\t\x12\x33\n\nattributes\x18\x03 \x03(\x0b\x32\x1f.com.rookout.Variant.NamedValue\x12\x13\n\tint_value\x18\x0b \x01(\x05H\x00\x12\x14\n\nlong_value\x18\x0c \x01(\x03H\x00\x12\x16\n\x0c\x64ouble_value\x18\r \x01(\x01H\x00\x12\x33\n\x0c\x62inary_value\x18\x0e \x01(\x0b\x32\x1b.com.rookout.Variant.BinaryH\x00\x12\x33\n\x0cstring_value\x18\x0f \x01(\x0b\x32\x1b.com.rookout.Variant.StringH\x00\x12/\n\nlist_value\x18\x10 \x01(\x0b\x32\x19.com.rookout.Variant.ListH\x00\x12-\n\tmap_value\x18\x11 \x01(\x0b\x32\x18.com.rookout.Variant.MapH\x00\x12\x39\n\x0fnamespace_value\x18\x13 \x01(\x0b\x32\x1e.com.rookout.Variant.NamespaceH\x00\x12)\n\x0b\x65rror_value\x18\x14 \x01(\x0b\x32\x12.com.rookout.ErrorH\x00\x12>\n\rmessage_value\x18\x17 \x01(\x0b\x32%.com.rookout.Variant.FormattedMessageH\x00\x12\x30\n\ntime_value\x18\x18 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x12\x35\n\ncode_value\x18\x1a \x01(\x0b\x32\x1f.com.rookout.Variant.CodeObjectH\x00\x12\x38\n\x0flarge_int_value\x18\x1b \x01(\x0b\x32\x1d.com.rookout.Variant.LargeIntH\x00\x12\x35\n\rcomplex_value\x18\x1c \x01(\x0b\x32\x1c.com.rookout.Variant.ComplexH\x00\x12\x36\n\nenum_value\x18\x1f \x01(\x0b\x32 .com.rookout.Variant.EnumerationH\x00\x1a?\n\nNamedValue\x12\x0c\n\x04name\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.com.rookout.Variant\x1a.\n\x06\x42inary\x12\x15\n\roriginal_size\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x0c\x1a.\n\x06String\x12\x15\n\roriginal_size\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\t\x1aQ\n\x04List\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x15\n\roriginal_size\x18\x02 \x01(\x05\x12$\n\x06values\x18\x03 \x03(\x0b\x32\x14.com.rookout.Variant\x1aQ\n\x04Pair\x12#\n\x05\x66irst\x18\x01 \x01(\x0b\x32\x14.com.rookout.Variant\x12$\n\x06second\x18\x02 \x01(\x0b\x32\x14.com.rookout.Variant\x1a\x46\n\x03Map\x12\x15\n\roriginal_size\x18\x01 \x01(\x05\x12(\n\x05pairs\x18\x02 \x03(\x0b\x32\x19.com.rookout.Variant.Pair\x1a@\n\tNamespace\x12\x33\n\nattributes\x18\x01 \x03(\x0b\x32\x1f.com.rookout.Variant.NamedValue\x1a#\n\x10\x46ormattedMessage\x12\x0f\n\x07message\x18\x01 \x01(\t\x1aL\n\nCodeObject\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06module\x18\x02 \x01(\t\x12\x10\n\x08\x66ilename\x18\x03 \x01(\t\x12\x0e\n\x06lineno\x18\x04 \x01(\r\x1a\x19\n\x08LargeInt\x12\r\n\x05value\x18\x01 \x01(\t\x1a*\n\x07\x43omplex\x12\x0c\n\x04real\x18\x01 \x01(\x01\x12\x11\n\timaginary\x18\x02 \x01(\x01\x1aM\n\x0b\x45numeration\x12\x14\n\x0cstring_value\x18\x01 \x01(\t\x12\x15\n\rordinal_value\x18\x02 \x01(\x05\x12\x11\n\ttype_name\x18\x03 \x01(\t\"\xd9\x03\n\x04Type\x12\x10\n\x0cVARIANT_NONE\x10\x00\x12\x0f\n\x0bVARIANT_INT\x10\x01\x12\x10\n\x0cVARIANT_LONG\x10\x02\x12\x12\n\x0eVARIANT_DOUBLE\x10\x03\x12\x12\n\x0eVARIANT_BINARY\x10\x04\x12\x12\n\x0eVARIANT_STRING\x10\x05\x12\x10\n\x0cVARIANT_LIST\x10\x06\x12\x0f\n\x0bVARIANT_MAP\x10\x07\x12\x12\n\x0eVARIANT_OBJECT\x10\x08\x12\x15\n\x11VARIANT_NAMESPACE\x10\t\x12\x11\n\rVARIANT_ERROR\x10\n\x12\x19\n\x15VARIANT_NOT_SUPPORTED\x10\x0b\x12\x15\n\x11VARIANT_MAX_DEPTH\x10\x0c\x12\x1d\n\x19VARIANT_FORMATTED_MESSAGE\x10\r\x12\x10\n\x0cVARIANT_TIME\x10\x0e\x12\x19\n\x15VARIANT_UKNOWN_OBJECT\x10\x0f\x12\x17\n\x13VARIANT_CODE_OBJECT\x10\x10\x12\x15\n\x11VARIANT_LARGE_INT\x10\x11\x12\x13\n\x0fVARIANT_COMPLEX\x10\x12\x12\x15\n\x11VARIANT_UNDEFINED\x10\x13\x12\x13\n\x0fVARIANT_DYNAMIC\x10\x14\x12\x10\n\x0cVARIANT_ENUM\x10\x15\x42\x07\n\x05valueb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])



_VARIANT_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='com.rookout.Variant.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='VARIANT_NONE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_INT', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_LONG', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_DOUBLE', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_BINARY', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_STRING', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_LIST', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_MAP', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_OBJECT', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_NAMESPACE', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_ERROR', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_NOT_SUPPORTED', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_MAX_DEPTH', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_FORMATTED_MESSAGE', index=13, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_TIME', index=14, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_UKNOWN_OBJECT', index=15, number=15,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_CODE_OBJECT', index=16, number=16,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_LARGE_INT', index=17, number=17,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_COMPLEX', index=18, number=18,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_UNDEFINED', index=19, number=19,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_DYNAMIC', index=20, number=20,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VARIANT_ENUM', index=21, number=21,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1799,
  serialized_end=2272,
)
_sym_db.RegisterEnumDescriptor(_VARIANT_TYPE)


_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='com.rookout.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='com.rookout.Error.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type', full_name='com.rookout.Error.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parameters', full_name='com.rookout.Error.parameters', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exc', full_name='com.rookout.Error.exc', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='traceback', full_name='com.rookout.Error.traceback', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=220,
)


_VARIANT_NAMEDVALUE = _descriptor.Descriptor(
  name='NamedValue',
  full_name='com.rookout.Variant.NamedValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='com.rookout.Variant.NamedValue.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.rookout.Variant.NamedValue.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1068,
  serialized_end=1131,
)

_VARIANT_BINARY = _descriptor.Descriptor(
  name='Binary',
  full_name='com.rookout.Variant.Binary',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='original_size', full_name='com.rookout.Variant.Binary.original_size', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.rookout.Variant.Binary.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1133,
  serialized_end=1179,
)

_VARIANT_STRING = _descriptor.Descriptor(
  name='String',
  full_name='com.rookout.Variant.String',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='original_size', full_name='com.rookout.Variant.String.original_size', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='com.rookout.Variant.String.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1181,
  serialized_end=1227,
)

_VARIANT_LIST = _descriptor.Descriptor(
  name='List',
  full_name='com.rookout.Variant.List',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='com.rookout.Variant.List.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='original_size', full_name='com.rookout.Variant.List.original_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='values', full_name='com.rookout.Variant.List.values', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1229,
  serialized_end=1310,
)

_VARIANT_PAIR = _descriptor.Descriptor(
  name='Pair',
  full_name='com.rookout.Variant.Pair',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='first', full_name='com.rookout.Variant.Pair.first', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second', full_name='com.rookout.Variant.Pair.second', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1312,
  serialized_end=1393,
)

_VARIANT_MAP = _descriptor.Descriptor(
  name='Map',
  full_name='com.rookout.Variant.Map',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='original_size', full_name='com.rookout.Variant.Map.original_size', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='pairs', full_name='com.rookout.Variant.Map.pairs', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1395,
  serialized_end=1465,
)

_VARIANT_NAMESPACE = _descriptor.Descriptor(
  name='Namespace',
  full_name='com.rookout.Variant.Namespace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributes', full_name='com.rookout.Variant.Namespace.attributes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1467,
  serialized_end=1531,
)

_VARIANT_FORMATTEDMESSAGE = _descriptor.Descriptor(
  name='FormattedMessage',
  full_name='com.rookout.Variant.FormattedMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='com.rookout.Variant.FormattedMessage.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1533,
  serialized_end=1568,
)

_VARIANT_CODEOBJECT = _descriptor.Descriptor(
  name='CodeObject',
  full_name='com.rookout.Variant.CodeObject',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='com.rookout.Variant.CodeObject.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='module', full_name='com.rookout.Variant.CodeObject.module', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filename', full_name='com.rookout.Variant.CodeObject.filename', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='lineno', full_name='com.rookout.Variant.CodeObject.lineno', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1570,
  serialized_end=1646,
)

_VARIANT_LARGEINT = _descriptor.Descriptor(
  name='LargeInt',
  full_name='com.rookout.Variant.LargeInt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='com.rookout.Variant.LargeInt.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1648,
  serialized_end=1673,
)

_VARIANT_COMPLEX = _descriptor.Descriptor(
  name='Complex',
  full_name='com.rookout.Variant.Complex',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='real', full_name='com.rookout.Variant.Complex.real', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='imaginary', full_name='com.rookout.Variant.Complex.imaginary', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1675,
  serialized_end=1717,
)

_VARIANT_ENUMERATION = _descriptor.Descriptor(
  name='Enumeration',
  full_name='com.rookout.Variant.Enumeration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='string_value', full_name='com.rookout.Variant.Enumeration.string_value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ordinal_value', full_name='com.rookout.Variant.Enumeration.ordinal_value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='type_name', full_name='com.rookout.Variant.Enumeration.type_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1719,
  serialized_end=1796,
)

_VARIANT = _descriptor.Descriptor(
  name='Variant',
  full_name='com.rookout.Variant',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='variant_type', full_name='com.rookout.Variant.variant_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='original_type', full_name='com.rookout.Variant.original_type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='attributes', full_name='com.rookout.Variant.attributes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='int_value', full_name='com.rookout.Variant.int_value', index=3,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='long_value', full_name='com.rookout.Variant.long_value', index=4,
      number=12, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='double_value', full_name='com.rookout.Variant.double_value', index=5,
      number=13, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='binary_value', full_name='com.rookout.Variant.binary_value', index=6,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='string_value', full_name='com.rookout.Variant.string_value', index=7,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='list_value', full_name='com.rookout.Variant.list_value', index=8,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_value', full_name='com.rookout.Variant.map_value', index=9,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='namespace_value', full_name='com.rookout.Variant.namespace_value', index=10,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error_value', full_name='com.rookout.Variant.error_value', index=11,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message_value', full_name='com.rookout.Variant.message_value', index=12,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='time_value', full_name='com.rookout.Variant.time_value', index=13,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='code_value', full_name='com.rookout.Variant.code_value', index=14,
      number=26, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='large_int_value', full_name='com.rookout.Variant.large_int_value', index=15,
      number=27, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='complex_value', full_name='com.rookout.Variant.complex_value', index=16,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='enum_value', full_name='com.rookout.Variant.enum_value', index=17,
      number=31, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_VARIANT_NAMEDVALUE, _VARIANT_BINARY, _VARIANT_STRING, _VARIANT_LIST, _VARIANT_PAIR, _VARIANT_MAP, _VARIANT_NAMESPACE, _VARIANT_FORMATTEDMESSAGE, _VARIANT_CODEOBJECT, _VARIANT_LARGEINT, _VARIANT_COMPLEX, _VARIANT_ENUMERATION, ],
  enum_types=[
    _VARIANT_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='com.rookout.Variant.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=223,
  serialized_end=2281,
)

_ERROR.fields_by_name['parameters'].message_type = _VARIANT
_ERROR.fields_by_name['exc'].message_type = _VARIANT
_ERROR.fields_by_name['traceback'].message_type = _VARIANT
_VARIANT_NAMEDVALUE.fields_by_name['value'].message_type = _VARIANT
_VARIANT_NAMEDVALUE.containing_type = _VARIANT
_VARIANT_BINARY.containing_type = _VARIANT
_VARIANT_STRING.containing_type = _VARIANT
_VARIANT_LIST.fields_by_name['values'].message_type = _VARIANT
_VARIANT_LIST.containing_type = _VARIANT
_VARIANT_PAIR.fields_by_name['first'].message_type = _VARIANT
_VARIANT_PAIR.fields_by_name['second'].message_type = _VARIANT
_VARIANT_PAIR.containing_type = _VARIANT
_VARIANT_MAP.fields_by_name['pairs'].message_type = _VARIANT_PAIR
_VARIANT_MAP.containing_type = _VARIANT
_VARIANT_NAMESPACE.fields_by_name['attributes'].message_type = _VARIANT_NAMEDVALUE
_VARIANT_NAMESPACE.containing_type = _VARIANT
_VARIANT_FORMATTEDMESSAGE.containing_type = _VARIANT
_VARIANT_CODEOBJECT.containing_type = _VARIANT
_VARIANT_LARGEINT.containing_type = _VARIANT
_VARIANT_COMPLEX.containing_type = _VARIANT
_VARIANT_ENUMERATION.containing_type = _VARIANT
_VARIANT.fields_by_name['variant_type'].enum_type = _VARIANT_TYPE
_VARIANT.fields_by_name['attributes'].message_type = _VARIANT_NAMEDVALUE
_VARIANT.fields_by_name['binary_value'].message_type = _VARIANT_BINARY
_VARIANT.fields_by_name['string_value'].message_type = _VARIANT_STRING
_VARIANT.fields_by_name['list_value'].message_type = _VARIANT_LIST
_VARIANT.fields_by_name['map_value'].message_type = _VARIANT_MAP
_VARIANT.fields_by_name['namespace_value'].message_type = _VARIANT_NAMESPACE
_VARIANT.fields_by_name['error_value'].message_type = _ERROR
_VARIANT.fields_by_name['message_value'].message_type = _VARIANT_FORMATTEDMESSAGE
_VARIANT.fields_by_name['time_value'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_VARIANT.fields_by_name['code_value'].message_type = _VARIANT_CODEOBJECT
_VARIANT.fields_by_name['large_int_value'].message_type = _VARIANT_LARGEINT
_VARIANT.fields_by_name['complex_value'].message_type = _VARIANT_COMPLEX
_VARIANT.fields_by_name['enum_value'].message_type = _VARIANT_ENUMERATION
_VARIANT_TYPE.containing_type = _VARIANT
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['int_value'])
_VARIANT.fields_by_name['int_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['long_value'])
_VARIANT.fields_by_name['long_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['double_value'])
_VARIANT.fields_by_name['double_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['binary_value'])
_VARIANT.fields_by_name['binary_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['string_value'])
_VARIANT.fields_by_name['string_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['list_value'])
_VARIANT.fields_by_name['list_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['map_value'])
_VARIANT.fields_by_name['map_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['namespace_value'])
_VARIANT.fields_by_name['namespace_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['error_value'])
_VARIANT.fields_by_name['error_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['message_value'])
_VARIANT.fields_by_name['message_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['time_value'])
_VARIANT.fields_by_name['time_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['code_value'])
_VARIANT.fields_by_name['code_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['large_int_value'])
_VARIANT.fields_by_name['large_int_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['complex_value'])
_VARIANT.fields_by_name['complex_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
_VARIANT.oneofs_by_name['value'].fields.append(
  _VARIANT.fields_by_name['enum_value'])
_VARIANT.fields_by_name['enum_value'].containing_oneof = _VARIANT.oneofs_by_name['value']
DESCRIPTOR.message_types_by_name['Error'] = _ERROR
DESCRIPTOR.message_types_by_name['Variant'] = _VARIANT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(
  DESCRIPTOR = _ERROR,
  __module__ = 'variant_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.Error)
  ))
_sym_db.RegisterMessage(Error)

Variant = _reflection.GeneratedProtocolMessageType('Variant', (_message.Message,), dict(

  NamedValue = _reflection.GeneratedProtocolMessageType('NamedValue', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_NAMEDVALUE,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.NamedValue)
    ))
  ,

  Binary = _reflection.GeneratedProtocolMessageType('Binary', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_BINARY,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.Binary)
    ))
  ,

  String = _reflection.GeneratedProtocolMessageType('String', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_STRING,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.String)
    ))
  ,

  List = _reflection.GeneratedProtocolMessageType('List', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_LIST,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.List)
    ))
  ,

  Pair = _reflection.GeneratedProtocolMessageType('Pair', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_PAIR,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.Pair)
    ))
  ,

  Map = _reflection.GeneratedProtocolMessageType('Map', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_MAP,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.Map)
    ))
  ,

  Namespace = _reflection.GeneratedProtocolMessageType('Namespace', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_NAMESPACE,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.Namespace)
    ))
  ,

  FormattedMessage = _reflection.GeneratedProtocolMessageType('FormattedMessage', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_FORMATTEDMESSAGE,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.FormattedMessage)
    ))
  ,

  CodeObject = _reflection.GeneratedProtocolMessageType('CodeObject', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_CODEOBJECT,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.CodeObject)
    ))
  ,

  LargeInt = _reflection.GeneratedProtocolMessageType('LargeInt', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_LARGEINT,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.LargeInt)
    ))
  ,

  Complex = _reflection.GeneratedProtocolMessageType('Complex', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_COMPLEX,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.Complex)
    ))
  ,

  Enumeration = _reflection.GeneratedProtocolMessageType('Enumeration', (_message.Message,), dict(
    DESCRIPTOR = _VARIANT_ENUMERATION,
    __module__ = 'variant_pb2'
    # @@protoc_insertion_point(class_scope:com.rookout.Variant.Enumeration)
    ))
  ,
  DESCRIPTOR = _VARIANT,
  __module__ = 'variant_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.Variant)
  ))
_sym_db.RegisterMessage(Variant)
_sym_db.RegisterMessage(Variant.NamedValue)
_sym_db.RegisterMessage(Variant.Binary)
_sym_db.RegisterMessage(Variant.String)
_sym_db.RegisterMessage(Variant.List)
_sym_db.RegisterMessage(Variant.Pair)
_sym_db.RegisterMessage(Variant.Map)
_sym_db.RegisterMessage(Variant.Namespace)
_sym_db.RegisterMessage(Variant.FormattedMessage)
_sym_db.RegisterMessage(Variant.CodeObject)
_sym_db.RegisterMessage(Variant.LargeInt)
_sym_db.RegisterMessage(Variant.Complex)
_sym_db.RegisterMessage(Variant.Enumeration)


# @@protoc_insertion_point(module_scope)



"""This is the external interface to the Rook package."""
import sys
import os
import traceback
import re

import six

from rook.config import ImportServiceConfig

from rook.serverless import on_lambda

_rook = None
_debug = False
_throw_errors = False

_TRUE_VALUES = ['y', 'Y', 'yes',  'Yes',  'YES', 'true', 'True', 'TRUE', '1', True]


def start(token=None,
          tags=None,
          host=None,
          port=None,
          debug=None,
          throw_errors=None,
          log_file=None,
          log_level=None,
          log_to_stderr=None,
          labels=None,
          use_import_hook=None,
          git_commit=None,
          git_origin=None,
          proxy=None,
          **kwargs):
    global _rook, _debug, _throw_errors

    if _rook is not None:
        return

    if isinstance(debug, bool):
        _debug = debug
    else:
        _debug = os.environ.get('ROOKOUT_DEBUG') in _TRUE_VALUES

    if isinstance(log_to_stderr, bool):
        log_to_stderr = log_to_stderr
    else:
        log_to_stderr = os.environ.get('ROOKOUT_LOG_TO_STDERR') in _TRUE_VALUES

    if isinstance(throw_errors, bool):
        _throw_errors = throw_errors

    if not isinstance(tags, list) and isinstance(os.environ.get('ROOKOUT_ROOK_TAGS'), six.string_types):
        raw_tags = os.environ.get('ROOKOUT_ROOK_TAGS')
        tags = [_normalize_string(tag.strip()) for tag in raw_tags.split(';') if tag]

    if not isinstance(labels, dict) and isinstance(os.environ.get('ROOKOUT_LABELS'), six.string_types):
        raw_labels = os.environ.get('ROOKOUT_LABELS')
        labels = {}
        for label in raw_labels.split(','):
            label = _normalize_string(label.strip())
            keyvalue = label.split(':')
            if len(keyvalue) == 2:
                key,value = keyvalue
                if key and value:
                    _validate_label(key)
                    labels[key] = value

    if on_lambda():
        log_file = ""
    log_file = log_file or os.environ.get('ROOKOUT_LOG_FILE')
    log_level = log_level or os.environ.get('ROOKOUT_LOG_LEVEL')
    host = host or os.environ.get('ROOKOUT_CONTROLLER_HOST') or os.environ.get('ROOKOUT_AGENT_HOST')
    port = port or os.environ.get('ROOKOUT_CONTROLLER_PORT') or os.environ.get('ROOKOUT_AGENT_PORT')
    proxy = proxy or os.environ.get('ROOKOUT_PROXY')
    token = token or os.environ.get('ROOKOUT_TOKEN')
    use_import_hook = use_import_hook if use_import_hook is not None else os.environ.get('ROOKOUT_USE_IMPORT_HOOK',
                                                                                         False) in _TRUE_VALUES
    async_start = os.environ.get('ROOKOUT_ASYNC_START', False) in _TRUE_VALUES

    try:
        from rook.exceptions.tool_exceptions import RookMissingToken, RookInvalidToken, RookVersionNotSupported, \
            RookCommunicationException, RookInvalidOptions, RookLoadError
        try:
            from rook.config import LoggingConfiguration, GitConfig

            if git_commit is not None:
                if not isinstance(git_commit, six.string_types):
                    raise RookInvalidOptions('git_commit should be a String')
                GitConfig.GIT_COMMIT = git_commit

            if git_origin is not None:
                if not isinstance(git_origin, six.string_types):
                    raise RookInvalidOptions('git_origin should be a String')
                GitConfig.GIT_ORIGIN = git_origin

            if log_file is not None:
                if not isinstance(log_file, six.string_types):
                    raise RookInvalidOptions('log_file should be a String')
                LoggingConfiguration.FILE_NAME = log_file

            if log_level is not None:
                if not isinstance(log_level, six.string_types):
                    raise RookInvalidOptions('log_level should be a String')
                LoggingConfiguration.LOG_LEVEL = log_level

            if log_to_stderr is not None:
                LoggingConfiguration.LOG_TO_STDERR = log_to_stderr

            if proxy is not None and not isinstance(proxy, six.string_types):
                raise RookInvalidOptions('proxy should be a String')

            if _debug:
                LoggingConfiguration.LOG_LEVEL = 'DEBUG'
                LoggingConfiguration.LOG_TO_STDERR = True
                LoggingConfiguration.DEBUG = True

            if use_import_hook is not None:
                ImportServiceConfig.USE_IMPORT_HOOK = use_import_hook

            if isinstance(tags, list):
                for tag in tags:
                    if not isinstance(tag, six.string_types):
                        raise RookInvalidOptions('Rook tags should be array of strings')
            else:
                if tags:
                    raise RookInvalidOptions('Rook tags should be array of strings')

            if not host and not token:
                raise RookMissingToken()
            else:
                if token is not None:
                    _validate_token(token)

            if (host == "staging.cloud.agent.rookout.com") or (host == "cloud.agent.rookout.com"):
                host = "https://" + host

            if (host == "staging.control.rookout.com") or (host == "control.rookout.com"):
                host = "wss://" + host

            import rook.singleton
            _rook = rook.singleton.singleton_obj

            return _rook.connect(token, host, port, proxy, tags, labels, async_start=async_start)
        except (RookMissingToken, RookInvalidToken, RookVersionNotSupported) as e:
            if not _throw_errors:
                six.print_("[Rookout] Failed to start Rookout:", e, file=sys.stderr)
            raise
        except RookCommunicationException as e:
            if not _throw_errors:
                six.print_("[Rookout] Failed to connect to the controller - will continue attempting in the background", e, file=sys.stderr)
            raise
        except ImportError as e:
            if not _throw_errors:
                six.print_("[Rookout] Failed to import dependencies:", e, file=sys.stderr)
            raise
        except (Exception, RookLoadError) as e:
            if not _throw_errors:
                six.print_("[Rookout] Failed initialization:", e, file=sys.stderr)
            raise
    except Exception:
        if _throw_errors:
            raise

        if _debug:
            traceback.print_exc()


def flush():
    global _rook
    if _rook is None:
        return

    _rook.flush()


def stop():
    global _rook
    if _rook is None:
        return

    _rook.stop()
    _rook = None


def _normalize_string(obj):
    if six.PY2:
        if isinstance(obj, str):
            return unicode(obj, errors="replace")
        else:
            return unicode(obj)
    else:
        return str(obj)


def _validate_token(token):
    from rook.exceptions import RookInvalidOptions

    if not isinstance(token, six.string_types):
        raise RookInvalidOptions('Rookout token should be a String')

    if len(token) != 64:
        raise RookInvalidOptions('Rookout token should be 64 characters')

    if re.match("^[0-9a-zA-Z]{0,64}$", token) is None:
        raise RookInvalidOptions('Rookout token must consist of only hexadecimal characters')


def _validate_label(label):
    from rook.exceptions import RookInvalidLabel

    if label.startswith("$"):
        raise RookInvalidLabel(label)



from __future__ import absolute_import

import unittest

import json

from rook import configuration


class TestConfigurationScheme(unittest.TestCase):

    def test_initialized_attribute(self):
        class TestScheme(configuration.ConfigurationScheme):
            A = 1

        TestScheme.A = 2

        self.assertEqual(2, TestScheme.A)


class TestConfigManager(unittest.TestCase):

    def test_load_dictionary(self):
        class Container(object):
            class TestScheme(configuration.ConfigurationScheme):
                A = 1
                B = 2

        c = configuration.ConfigManager(Container)

        c.load_dictionary({'TestScheme' : {'A' : 4, 'B': 5}})

        self.assertEqual(4, Container.TestScheme.A)
        self.assertEqual(5, Container.TestScheme.B)

    def test_load_json_file(self):
        class Container(object):
            class TestScheme(configuration.ConfigurationScheme):
                A = 1
                B = 2

        with open('test.json', 'w') as f:
            json.dump({'TestScheme' : {'A' : 4, 'B': 5}}, f)

        c = configuration.ConfigManager(Container)
        c.load_json_file('test.json')

        self.assertEqual(4, Container.TestScheme.A)
        self.assertEqual(5, Container.TestScheme.B)

    def test_load_class(self):
        class Container(object):
            class TestScheme(configuration.ConfigurationScheme):
                class TestScheme(configuration.ConfigurationScheme):
                    A = 1
                    B = 2

        class TestScheme(configuration.ConfigurationValues):
            class TestScheme(configuration.ConfigurationValues):
                A = 4
                B = 5

        c = configuration.ConfigManager(Container)
        c.load_config_class(TestScheme)

        self.assertEqual(4, Container.TestScheme.TestScheme.A)
        self.assertEqual(5, Container.TestScheme.TestScheme.B)

    def test_load_module(self):
        class Container(object):
            class TestScheme(configuration.ConfigurationScheme):
                A = 1
                B = 2

        class ValuesContainer(object):
            class TestScheme(configuration.ConfigurationValues):
                A = 4
                B = 5

        c = configuration.ConfigManager(Container)
        c.load_module(ValuesContainer)

        self.assertEqual(4, Container.TestScheme.A)
        self.assertEqual(5, Container.TestScheme.B)

    def test_dump_json_file(self):
        class Container(object):
            class TestScheme(configuration.ConfigurationScheme):
                A = 1
                B = 2

        Container.TestScheme.A = 4
        Container.TestScheme.B = 5

        c = configuration.ConfigManager(Container)
        c.dump_json_file(['TestScheme'], 'test.json')

        with open('test.json', "r") as f:
            contents = json.load(f)

        self.assertEqual({'TestScheme': {'A': 4, 'B': 5}}, contents)

    def test_dump_json_file_all(self):
        class Container(object):
            class Config1(configuration.ConfigurationScheme):
                A = 1
            class Config2(configuration.ConfigurationScheme):
                B = 2

        c = configuration.ConfigManager(Container)
        c.dump_json_file_all('test_all.json')

        with open('test_all.json', "r") as f:
            contents = json.load(f)

        self.assertEqual({'Config1': {'A': 1}, 'Config2': {'B': 2}}, contents)

    def test_dump_and_load_json_file(self):
        class Container1(object):
            class TestScheme(configuration.ConfigurationScheme):
                A = 1
                B = 2

        class Container2(object):
            class TestScheme(configuration.ConfigurationScheme):
                A = 1
                B = 2

        Container1.TestScheme.A = 4
        Container1.TestScheme.B = 5

        c1 = configuration.ConfigManager(Container1)
        c1.dump_json_file(['TestScheme'], 'test.json')

        c2 = configuration.ConfigManager(Container2)
        c2.load_json_file('test.json')

        self.assertEqual(4, Container1.TestScheme.A)
        self.assertEqual(5, Container2.TestScheme.B)



import datetime

from .dumped_object_namespace import DumpedObjectNamespace

from .python_object_namespace import PythonObjectNamespace


class DumpedPrimitiveNamespace(DumpedObjectNamespace):

    def __init__(self, obj, type, common_type, attributes={}, methods=()):
        super(DumpedPrimitiveNamespace, self).__init__(type, common_type, attributes, methods)
        self.obj = obj

    def _get_value_for_json(self):
        if isinstance(self.obj, complex):
            return {u'real': self.obj.real, u'imag': self.obj.imag}
        elif isinstance(self.obj, datetime.datetime):
            return str(self.obj)
        else:
            return self.obj

    def to_dict(self):
        return {
            u'@namespace': self.__class__.__name__,
            u'@common_type': self.common_type,
            u'@original_type': self.type,
            u'@attributes' : self.get_attributes_dict(),
            u'@value': self._get_value_for_json(),
            }

    def to_simple_dict(self):
        return self._get_value_for_json()

    def __hash__(self):
        return hash(self.obj)



import sys
import os
import platform
import socket

import six

from rook.config import VersionConfiguration, GitConfig
from rook import git
import rook.protobuf2.agent_info_pb2 as agent_info_pb

from rook.logger import logger


class Information(object):
    def collect(self):
        for name, collector in six.iteritems(self._collectors):
            try:
                if callable(collector):
                    value = collector()
                else:
                    value = collector
                setattr(self, name, value)
            except Exception as exc:
                logger.debug("Failed to collect %s information: %s", name, exc)

                setattr(self, name, "")
                pass

        return self


class SCMInformation(Information):
    def __init__(self):
        self._collectors = {
            'commit': self._get_commit,
            'origin': self._get_origin
        }

    def _get_commit(self):
        user_commit = GitConfig.GIT_COMMIT or os.environ.get('ROOKOUT_COMMIT', '')

        if user_commit:
            return user_commit
        else:
            git_root = self._get_git_root()

            if git_root:
                return git.get_revision(git_root)

        return ''

    def _get_origin(self):
        user_remote_origin = GitConfig.GIT_ORIGIN or os.environ.get('ROOKOUT_REMOTE_ORIGIN', '')

        if user_remote_origin:
            return user_remote_origin
        else:
            git_root = self._get_git_root()

            if git_root:
                return git.get_remote_origin(git_root)

        return ''

    def _get_git_root(self):
        return os.environ.get('ROOKOUT_GIT') or git.find_root(os.path.dirname(os.path.abspath(sys.argv[0])))


class NetworkInformation(Information):
    def __init__(self):
        self._collectors = {
            'ip_addr': self._get_ip_addr,
            'network': platform.node()
        }

    @staticmethod
    def _get_ip_addr():
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('10.255.255.255', 1))
        return s.getsockname()[0]


class SystemInformation(Information):
    def __init__(self):
        self._collectors = {
            'hostname': socket.gethostname,
            'os': platform.system,
            'os_version': self._get_os_version,
            # linux distro name on linux, blank otherwise
            'distro': lambda: platform.system() == 'Linux' and platform.linux_distribution()[0] or '',
            'arch': platform.machine()
        }

    def _get_os_version(self):
        system = platform.system()

        if system == 'Darwin':
            version = platform.mac_ver()[0]
        elif system == 'Linux':
            version = platform.linux_distribution()[1]
        elif system == 'Windows':
            version = platform.win32_ver()[0]
        else:
            version = ''

        return version


class VersionInformation(Information):
    def __init__(self):
        self._collectors = {
            'version': VersionConfiguration.VERSION,
            'commit': VersionConfiguration.COMMIT
        }


class PlatformInformation(Information):
    def __init__(self):
        self._collectors = {
            'platform': 'python',
            'version': sys.version,
            'variant': platform.python_implementation
        }


class AgentInformation(Information):
    def __init__(self):
        self._collectors = {
            'version': lambda: VersionInformation().collect(),
            'network': lambda: NetworkInformation().collect(),
            'system': lambda: SystemInformation().collect(),
            'platform': lambda: PlatformInformation().collect(),
            'scm': lambda: SCMInformation().collect(),
            'executable': lambda: sys.argv[0],
            'command_arguments': lambda: sys.argv[1:],
            'process_id': os.getpid
        }


def collect():
    return AgentInformation().collect()


def pack_agent_info(info):
    packed_info = agent_info_pb.AgentInformation()
    packed_info.agent_id = info.agent_id

    packed_info.version.CopyFrom(agent_info_pb.VersionInformation())
    packed_info.version.version = info.version.version
    packed_info.version.commit = info.version.commit

    packed_info.network.CopyFrom(agent_info_pb.NetworkInformation())
    packed_info.network.ip_addr = info.network.ip_addr
    packed_info.network.network = info.network.network

    packed_info.system.CopyFrom(agent_info_pb.SystemInformation())
    packed_info.system.hostname = info.system.hostname
    packed_info.system.os = info.system.os
    packed_info.system.os_version = info.system.os_version
    packed_info.system.distro = info.system.distro
    packed_info.system.arch = info.system.arch

    packed_info.platform.CopyFrom(agent_info_pb.PlatformInformation())
    packed_info.platform.platform = info.platform.platform
    packed_info.platform.version = info.platform.version
    packed_info.platform.variant = info.platform.variant

    packed_info.scm.CopyFrom(agent_info_pb.SCMInformation())
    packed_info.scm.commit = info.scm.commit
    packed_info.scm.origin = info.scm.origin

    packed_info.executable = info.executable
    packed_info.command_arguments.extend(info.command_arguments)

    packed_info.process_id = info.process_id

    for label_key, label_value in six.iteritems(info.labels):
        packed_info.labels[label_key] = label_value

    packed_info.tags.extend(info.tags)

    return packed_info



"""This module is in charge of managing the rook's module state.

The external interface for the module is Rook located in rook.rook."""

import os
import sys
import platform
import atexit

# This must be the first import, otherwise enable_gevent_for_grpc will fail since grpc has already been imported
from rook.logger import logger

from rook.config import ControllerAddress, AgentComConfiguration

from .augs.augs_manager import AugsManager
from .trigger_services import TriggerServices

from rook.exceptions import RookInterfaceException, RookVersionNotSupported

from .atfork import install_fork_handler


class _Singleton(object):
    """This is singleton is the class managing the module.

    It should never be referred to directly, instead use obj in this module."""

    def __init__(self):
        """Initialize the object, sets member variables."""
        self._check_version_supported()

        logger.info("Initializing Rook under process-%d", os.getpid())

        self._services_started = False

        self._trigger_services = TriggerServices()
        self._aug_manager = None
        self._fork_handler = None
        self._output = None

        self._agent_com = None

    def _start_trigger_services(self):
        """Start trigger services.

        Calling this method multiple times will have no effect.
        """
        # Don't double init services
        if self._services_started:
            return

        if self._output is None:
            from rook.com.output import Output
            self._output = Output(self._trigger_services)

        self._trigger_services.start()
        self._aug_manager = AugsManager(self._trigger_services, self._output)
        self._services_started = True

    def _stop_trigger_services(self):
        if not self._services_started:
            return

        self._aug_manager = None
        self._trigger_services.close()

        self._services_started = False

    def get_trigger_services(self):
        return self._trigger_services

    def connect(self, token, host, port, proxy, tags=None, labels=None, async_start=False):
        """Connect to the Agent."""
        if self._agent_com:
            raise RookInterfaceException("Multiple connection attempts not supported!")

        self._fork_handler = install_fork_handler()

        if not host:
            host = ControllerAddress.HOST

        if not port:
            port = int(ControllerAddress.PORT)

        if not token:
            token = os.environ.get('ROOKOUT_TOKEN')

        logger.debug("Initiating AgentCom-\t%s:%d", host, int(port))
        if host.startswith('ws://') or host.startswith('wss://'):
            from rook.protobuf2 import variant_pb2
            import rook.processor.namespace_serializer
            rook.processor.namespace_serializer.rook_pb2 = variant_pb2

            from rook.com_ws.agent_com_ws import AgentCom
            from rook.com_ws.command_handler import CommandHandler
            from rook.com_ws.output_ws import Output

            labels = labels or {}
            tags = tags or []

            self._output = Output()
            self._agent_com = AgentCom(host, port, proxy, token, labels, tags)
            self._start_trigger_services()
            self._command_handler = CommandHandler(self._agent_com, self._aug_manager)
            self._output.set_agent_com(self._agent_com)
            self._agent_com.start()
            if async_start is False:
                self._agent_com.wait_for_ready(AgentComConfiguration.TIMEOUT)
        else:
            from .com.agent_com import AgentCom
            from rook.com.output import Output

            if async_start is True:
                AgentComConfiguration.TIMEOUT = 0

            self._output = Output(self._trigger_services)

            if tags is not None:
                self._output.tags = tags
            if labels is not None:
                self._output.labels = labels

            self._start_trigger_services()
            self._agent_com = AgentCom(self._aug_manager, self._trigger_services, self._output, host, port, token)
            self._output.set_agent_com(self._agent_com)
            self._agent_com.start()


    logger.info("Successful connection to agent")

    def flush(self):
        self._output.flush_messages()

    def stop(self):
        logger.debug("Shutting down")

        if self._agent_com is not None:
            self._agent_com.stop()

        self._stop_trigger_services()

        self._agent_com = None

    @staticmethod
    def _check_version_supported():
        try:
            supported_platforms = ['pypy', 'cpython']
            supported_version = ['2.7', '3.5', '3.6', '3.7']

            current_platform = platform.python_implementation().lower()
            if current_platform not in supported_platforms:
                raise RookVersionNotSupported("Rook is not supported in this platform: " + current_platform)

            major, minor, _, _, _ = sys.version_info
            current_version = "{}.{}".format(major, minor)
            if current_version not in supported_version:
                raise RookVersionNotSupported("Rook is not supported in this python version: " + current_version)

        except Exception as e:
            import traceback
            traceback.print_exc()

            raise e


singleton_obj = _Singleton()


def exit_handler():
    try:
        logger.info("Exit handler called - flushing")
        if singleton_obj._agent_com:
            singleton_obj.flush()
    except:
        logger.exception("Flush failed")
    logger.info("Exit handler finished")


atexit.register(exit_handler)



from .bdb_location_service import BdbLocationService
from .import_service import ImportService
from .logging_location_service import LoggingLocationService
from .http_server_services.http_server_services import HttpServerService



import os
import six
import json


class ConfigurationSchemeMetaClass(type):
    def __init__(cls, name, bases, dct):
        super(ConfigurationSchemeMetaClass, cls).__init__(name, bases, dct)

    def __setattr__(self, key, value):
        if key not in type.__getattribute__(self, '__dict__'):
            raise AttributeError("Attribute does not exists in scheme!", key)
        else :
            type.__setattr__(self, key, value)


@six.add_metaclass(ConfigurationSchemeMetaClass)
class ConfigurationScheme(object):
    pass


class ConfigurationValues(object):
    pass


class ConfigManager(object):

    def __init__(self, container):
        self.container = container

    def load_dictionary(self, dictionary):
        self._load_dictionary_recursive(self.container, dictionary)

    def load_config_class(self, cls, by_name=None):
        self._load_object_recursive(self.container.__dict__[by_name or cls.__name__], cls)

    def load_json_file(self, path, namespace=None):
        with open(path, "r") as f:
            contents = json.load(f)

        if isinstance(contents, list):
            for obj in contents:
                self.load_dictionary(obj)
        elif isinstance(contents, dict):
            self.load_dictionary(contents)
        else:
            raise ValueError("Unknown JSON object type")

    # TODO - test this method
    def safe_load_json_file(self, path, namespace=None):
        try:
            if os.path.exists(path):
                self.load_json_file(path, namespace)
        except Exception:
            import logging
            logging.exception("Failed to load configuration json file- %s", path)

    def load_module(self, module):
        for key, value in six.iteritems(module.__dict__):
            if isinstance(value, type) and issubclass(value, ConfigurationValues) and value is not ConfigurationValues:
                self._load_object_recursive(self.container.__dict__[key], value)

    def dump_json_file(self, names, path):
        configuration = {}

        for name in names:
            configuration[name] = ConfigManager._dump_object_recursive(self.container.__dict__[name])

        with open(path, "w") as f:
            json.dump(configuration, f)

    def dump_json_file_all(self, path):
        configuration = {}

        for key, value in six.iteritems(self.container.__dict__):
            if isinstance(value, ConfigurationSchemeMetaClass):
                configuration[key] = ConfigManager._dump_object_recursive(value)

        with open(path, "w") as f:
            json.dump(configuration, f)

    def _load_dictionary_recursive(self, node, dictionary):
        for key, value in six.iteritems(dictionary):
            if isinstance(value, dict):
                self._load_dictionary_recursive(node.__dict__[key], value)
            else:
                setattr(node, key, value)

    def _load_object_recursive(self, node, obj):
        for key, value in six.iteritems(obj.__dict__):
            if isinstance(value, type) and issubclass(value, ConfigurationValues):
                self._load_object_recursive(node.__dict__[key], value)
            else:
                setattr(node, key, value)

    @staticmethod
    def _dump_object_recursive(obj):
        dictionary = {}

        for key, value in six.iteritems(obj.__dict__):
            if not ConfigManager._is_key_built_in(key):
                if isinstance(value, ConfigurationScheme):
                    dictionary[key] = ConfigManager._dump_object_recursive(value)
                else:
                    dictionary[key] = value

        return dictionary

    @staticmethod
    def _is_key_built_in(key):
        return key in ('__builtins__', '__file__', '__package__', '__name__', '__doc__', '__dict__', '__weakref__',
                       '__module__', '__spec__', '__loader__', '__cached__', '_initialized')
